----------------------------------------------------------------------------------
-- Engineer: Nazim Aliev <alnaz@mail.ru>
-- 
-- Description: Testbench module for simulation 
--
-- Project: OV7670 Camera data processing with Zedboard FPGA
--
-- Web: http://www.nazim.ru/2512
----------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity tb_top is
	--  Port ( );
	end tb_top;

architecture Behavioral of tb_top is

	-- only in ports, no need out
	component ov7670_top
		Port (
					 clk100_zed   : in    std_logic;
					 btn         : in    std_logic;
					 ov7670_pclk : in std_logic;
					 ov7670_vsync : in std_logic;
					 ov7670_href  : in std_logic;
					 ov7670_d     : in std_logic_vector(7 downto 0));
	end component;

	constant num_cycles : integer := 80000;
	signal clkgen: std_logic := '0';
	signal btn: std_logic := '0';
	signal pclk: std_logic := '0';
	signal vsync: std_logic := '0';
	signal href : std_logic := '0';
	signal d: std_logic_vector(7 downto 0) := "00000000";
	signal counter  : integer := 0;


begin

	tb_top: ov7670_top
	port map(
						clk100_zed   => clkgen,
						btn => btn,
						ov7670_pclk => clkgen,
						ov7670_vsync => vsync,
						ov7670_href => href,
						ov7670_d => d);

	process
	begin --process      
		for i in 1 to num_cycles loop
			clkgen <= not clkgen;
			wait for 5 ns;
			clkgen <= not clkgen;
			wait for 5 ns;
		end loop;
		wait;  -- simulation stops here
	end process;
	
	process(pclk)
	begin
		if rising_edge(pclk) then  
			d <= std_logic_vector(unsigned(d) + 1);
		end if;
	end process;
			
	process
	begin --process
		wait for 100 ps;
		btn <= '1';
		wait;  -- simulation stops here
	end process;

end Behavioral;

